# @push.rocks/smartbucket
simple cloud independent object storage

## Install

To install `@push.rocks/smartbucket`, you need to have Node.js and npm (Node Package Manager) installed on your system. If you have them installed, you can add `@push.rocks/smartbucket` to your project by running the following command in your project's root directory:

```bash
npm install @push.rocks/smartbucket --save
```

This command will download and install `@push.rocks/smartbucket` along with its required dependencies into your project's `node_modules` directory and save it as a dependency in your project's `package.json` file.

## Usage

`@push.rocks/smartbucket` is a module designed to provide simple cloud-independent object storage functionality. It wraps various cloud storage providers such as AWS S3, Google Cloud Storage, and others, offering a unified API to manage storage buckets and objects within those buckets.

To use `@push.rocks/smartbucket` in your project, you'll need to follow these general steps:

### Setting Up

First, ensure you are using ECMAScript modules (ESM) and TypeScript in your project for best compatibility. 

Here's how to import and initialize smartbucket in a TypeScript file:

```typescript
import {
  SmartBucket,
  Bucket,
  Directory,
  File
} from '@push.rocks/smartbucket';

const mySmartBucket = new SmartBucket({
  accessKey: "yourAccessKey",
  accessSecret: "yourSecretKey",
  endpoint: "yourEndpointURL",
});
```

Make sure to replace `"yourAccessKey"`, `"yourSecretKey"`, and `"yourEndpointURL"` with your actual credentials and endpoint URL.

### Creating a New Bucket

To create a new bucket:

```typescript
async function createBucket(bucketName: string) {
  try {
    const myBucket: Bucket = await mySmartBucket.createBucket(bucketName);
    console.log(`Bucket ${bucketName} created successfully.`);
  } catch (error) {
    console.error("Error creating bucket:", error);
  }
}

// Use the function
createBucket("exampleBucket");
```

**Important:** Bucket names must be unique across the storage service.

### Listing Buckets

To list all buckets:

```typescript
// Currently, SmartBucket does not include a direct method to list all buckets,
// but you can access the underlying client provided by the cloud storage SDK
// to perform such operations, depending on the SDK's capabilities.
```

### Uploading Objects to a Bucket

To upload an object to a bucket:

```typescript
async function uploadFile(bucketName: string, filePath: string, fileContent: Buffer | string) {
  const myBucket: Bucket = await mySmartBucket.getBucketByName(bucketName);
  if (myBucket) {
    await myBucket.fastStore(filePath, fileContent);
    console.log(`File uploaded to ${bucketName} at ${filePath}`);
  }
}

// Use the function
uploadFile("exampleBucket", "path/to/object.txt", "Hello, world!");
```

### Downloading Objects from a Bucket

To download an object:

```typescript
async function downloadFile(bucketName: string, filePath: string) {
  const myBucket: Bucket = await mySmartBucket.getBucketByName(bucketName);
  if (myBucket) {
    const fileContent: Buffer = await myBucket.fastGet(filePath);
    console.log("Downloaded file content:", fileContent.toString());
  }
}

// Use the function
downloadFile("exampleBucket", "path/to/object.txt");
```

### Deleting Objects

To delete an object from a bucket:

```typescript
async function deleteFile(bucketName: string, filePath: string) {
  const myBucket: Bucket = await mySmartBucket.getBucketByName(bucketName);
  if (myBucket) {
    await myBucket.fastRemove(filePath);
    console.log(`File at ${filePath} deleted from ${bucketName}.`);
  }
}

// Use the function
deleteFile("exampleBucket", "path/to/object.txt");
```

### Working with Directories

`@push.rocks/smartbucket` abstracts directories within buckets for easier object management. You can create, list, and delete directories using the `Directory` class.

### Additional Features

- **Bucket Policies:** Manage bucket policies to control access permissions.
- **Object Metadata:** Retrieve and modify object metadata.
- **Cloud-Agnostic:** Designed to work with multiple cloud providers, allowing for easier migration or multi-cloud strategies.

Remember, each cloud provider has specific features and limitations. `@push.rocks/smartbucket` aims to abstract common functionalities, but always refer to the specific cloud provider's documentation for advanced features or limitations.

> **Note:** This document focuses on basic operations to get you started with `@push.rocks/smartbucket`. For advanced usage, including streaming data, managing bucket policies, and handling large file uploads, refer to the detailed API documentation and examples.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
